#include "heavy_blaster.h"
#include "world_entities/projectiles/projectile.h"

#include "world_entity.h"
#include "static_model.h"
#include "weapon_manager.h"
#include "util/loading/factory.h"

#include "animation3d.h"

#include "loading/fast_factory.h"

CREATE_FACTORY(HeavyBlaster);
/**
 * Standard constructor
 */
HeavyBlaster::HeavyBlaster ()
 : Weapon()
{
    this->init();
}

HeavyBlaster::HeavyBlaster (const TiXmlElement* root = NULL)
 : Weapon()
{
    this->init();
    if (root != NULL)
      this->loadParams(root);
}

/**
 * Default destructor
 */
HeavyBlaster::~HeavyBlaster()
{
      // model will be deleted from WorldEntity-destructor
}

void HeavyBlaster::loadParams(const TiXmlElement* root)
{
  Weapon::loadParams(root);
}

void HeavyBlaster::init()
{
  //this->registerObject(this, HeavyBlaster::_objectList);

//  this->model = (Model*)ResourceManager::getInstance()->load("models/guns/test_gun.obj", OBJ, RP_CAMPAIGN);

  this->loadModel("models/guns/plasmadriver_#.obj", 1.0);
  

  this->setStateDuration(WS_SHOOTING, 0.05);
  this->setStateDuration(WS_RELOADING, 0);
  this->setStateDuration(WS_ACTIVATING, .5);
  this->setStateDuration(WS_DEACTIVATING, 1);

  this->setEnergyMax(5000);
  this->increaseEnergy(5000);
  //this->minCharge = 2;

  this->setActionSound(WA_SHOOT, "sound/laser.wav");
  this->setActionSound(WA_ACTIVATE, "sound/voices/lasers.wav");
  this->setActionSound(WA_RELOAD, "sound/spawn/alien_generator.wav");

  this->setCapability(WTYPE_ALLDIRS | WTYPE_DIRECTIONAL | WTYPE_LIGHT);
  this->setProjectileTypeC("RailProjectile");   // FIXME temp project type until the blaste class exist
  this->prepareProjectiles(100);

  Animation3D* animation2 = this->getAnimation(WS_ACTIVATING, this);
  Animation3D* animation3 = this->getAnimation(WS_DEACTIVATING, this);

  animation2->setInfinity(ANIM_INF_CONSTANT);
  animation3->setInfinity(ANIM_INF_CONSTANT);

  this->setEmissionPoint(3.8, 1.2, 0);

  animation2->addKeyFrame(Vector(0.0, -1.0, 0.0), Quaternion(), 0.3, ANIM_LINEAR, ANIM_NULL);
  animation2->addKeyFrame(Vector(0.0, 0.0, 0.0), Quaternion(), 0.5, ANIM_LINEAR, ANIM_NULL);

  animation3->addKeyFrame(Vector(0.0, 0.0, 0.0), Quaternion(), 0.5, ANIM_LINEAR, ANIM_NULL);
  animation3->addKeyFrame(Vector(0.0, -1.0, 0.0), Quaternion(), 0.3, ANIM_LINEAR, ANIM_NULL);
}


void HeavyBlaster::fire()
{
  Projectile* pj =  this->getProjectile();
  if (pj == NULL)
    return;

  // set the owner
  pj->setOwner(this->getOwner());

  pj->setParent(PNode::getNullParent());

  pj->setVelocity(this->getAbsDir().apply(Vector(1,0,0))*250 + VECTOR_RAND(5));

  pj->setAbsCoor(this->getEmissionPoint());
  pj->setAbsDir(this->getAbsDir());
  pj->activate();
}

/**
 *  this activates the weapon
*/
void HeavyBlaster::activate()
{
}

/**
 *  this deactivates the weapon
*/
void HeavyBlaster::deactivate()
{
}

void HeavyBlaster::draw() const
{
}
